#/usr/bin/python

"""
This is a script to migrate users from LDAP to Plone internal user management.
I know this is damn ugly, but I needed to make some tweaks to bypass Zope's security policies, i.e. importing other classes
@author: Katrin Krieger <kkrieger@iws.cs.uni-magdeburg.de>
"""

import ldap
import sys
#from Products.CMFCore.utils import getToolByName
#from zLOG import LOG, INFO, ERROR
from ldap.cidict import cidict



def ldap_migration(self):    
    #REQUEST = context.REQUEST
    #RESPONSE = REQUEST.RESPONSE 
    #regtool = getToolByName(self, 'portal_registration')
    #grouptool = getToolByName(self, 'portal_groups') 
    index = 1
    imported_count = 0
    
    user_dn = None
    user_pw = None
    dump_dn = None
    
	#hatte einen Dump vom WDOK gezogen und in einen lokalen LDAP eingespielt, so dass ich ihn Ruhe rumprobieren konnte
    server = 'ldap://localhost'
    #filter = '(objectclass=*)'
    filter = 'cn=*';
    #attrs = ['*']
    attrs = None
    
    #usage="""Usage: %s user_dn dn
    
    #Log in as user_dn and dump the record for person dn.n\n """ % sys.argv[0]
    
    #if len(sys.argv) != 3:
    #    sys.stderr.write("Error: expected user_dn and dn.n\n")
    #    sys.stdout.write(usage)
    #    sys.exit(1)
    
    user_dn = "cn=Manager,dc=iws,dc=cs,dc=uni-magdeburg,dc=de"
    dump_dn = "dc=iws,dc=cs,dc=uni-magdeburg,dc=de"
    user_pw = "Asdf,." #yep i am lazy and don't want to type in the password everytime i want to start this script
	
    # Add a blank line...
    sys.stdout.write("\n")
    
    try:
        l = ldap.initialize(server)
        try:
            #l.start_tls_s()
            l.bind(user_dn, user_pw)
            result_set = []
            res = []
            timeout = 0
            raw_res = l.search( dump_dn, ldap.SCOPE_SUBTREE, filter,
                attrs )
            while 1:
                result_type, result_data = l.result(raw_res, timeout)
                if (result_data == []):
                    break
                else:
                    if result_type == ldap.RES_SEARCH_ENTRY:
                        result_set.append(result_data)
    
           
            if(raw_res):
                res = get_search_results( result_set )
            else:
                print "no raw_res"
              
            #record has attributes attrs and dn
            print res
            if res and not(res==[]):
              
                for record in res:
                    #do we have a user or a group? we handle users first
                    if ('OpenLDAPperson' in record.attrs['objectclass']):
                        # check if the desired attribute exists
                        
                        properties = {}
                        try:    
                            properties['user_name'] = record.attrs['uid']
                        except KeyError: 
                            print "No such key: uid"
                            properties['user_name'] = ""
                        try:
                            properties['fullname'] = record.attrs['cn']
                        except KeyError: 
                            print "No such key: cn"
                            properties['cn'] = ""
                        try:
                            properties['givenName']=record.attrs['givenName']
                        except KeyError: 
                            print "No such key: givenName"
                            properties['givenName'] = ""
                        try:
                            properties['email'] = record.attrs['mail']
                        except KeyError: 
                            print "No such key: mail"
                            properties['email'] = ""
                        try:
                            properties['sn'] = record.attrs['sn']
                        except KeyError: 
                            print "No such key: sn"
                            properties['sn'] = ""
                        try:
                            properties['major'] = record.attrs['departmentNumber']
                        except KeyError: 
                            print "No such key: departmentNumber"
                            properties['major'] = ""
                        try:
                            properties['personal_title'] = record.attrs['personalTitle']
                        except KeyError: 
                            print "No such key: personalTitle"
                            properties['personal_title'] = ""
                        try:
                            properties['student_id'] = record.attrs['employeeNumber']
                        except KeyError: 
                            print "No such key: employeeNumber"
                            properties['student_id'] = ""
                            
                        print properties
                        print "\n\n"
                   
                    try:
                        #regtool.addMember(record.attrs['uid'],record.attrs['userpassword'], properties=properties)
                        #print "Successfully added %s %s (%s) with email %s" % (first, last, id, email)
                        #LOG('ldap_migration', INFO, "Successfully added %s with email %s" % (id, record.attrs['mail']))
                        imported_count += 1
                    except ValueError, e:
                        pass
                        #LOG('ldap_migration', ERROR, "Could not add %s with email %s: %s" % (id, record.attrs['mail'],e))
                    
            print "Imported %s users to Plone" % (imported_count)
          
            #second loop for adding users to groups
            for record in res:
                if ('groupOfUniqueNames' in record.attrs['objectclass']):
                    #extract group name
                    #groupname = record.attrs['cn'][0]
                    #grp = grouptool.getGroupById(groupname)
                    #fetch users
                    listOfUsersOfGroup = record.attrs['uniquemember']
                    for users in listOfUsersOfGroup:
                        #we just want the uid, not cn
                        if not(users.startswith("cn")):
                            print "adding user %s to group %s " % ((users.partition(",")[0]).partition("=")[2], groupname)
                            #grp.addMember((users.partition(",")[0]).partition("=")[0])
                else:
                    pass
    
        except ldap.INVALID_CREDENTIALS:
            print "Your username or password is incorrect."
            sys.exit()
        except ldap.LDAPError, e:
            if type(e.message) == dict:
                for (k, v) in e.message.iteritems():
                    sys.stderr.write("%s: %sn" % (k, v) )
            else:
                sys.stderr.write(e.message)
            sys.exit(1)
    
    finally:
        l.unbind()
   
def get_search_results(results):
    """Given a set of results, return a list of LDAPSearchResult
    objects.
    """
   
    res = []
    dn = ''
    if type(results) == tuple and len(results) == 2 :
       # LOG('ldap_migration', ERROR, "foobar")  
        (code, arr) = results
    elif type(results) == list:
        arr = results

    if len(results) == 0:
        return res

    for item in arr:
        (dn, attrs) = item[0]
        cidict(attrs)
        res.append( (dn,attrs) )
        
def main():
    ldap_migration("foo");

if __name__ == "__main__":
    main()




# -*- coding: utf-8 -*-
# $Id: interfaces.py 1542 2011-04-01 07:13:53Z amelung $
#
# Copyright (c) 2006-2011 Otto-von-Guericke-UniversitŠt Magdeburg
#
# This file is part of ECLecture.
#
__author__ = """Mario Amelung <mario.amelung@gmx.de>"""
__docformat__ = 'plaintext'

from zope.interface import Interface

class IECLecture(Interface):
    """
    Marker interface for .ECLecture.ECLecture
    """



# -*- coding: utf-8 -*-
# $Id: validators.py 1542 2011-04-01 07:13:53Z amelung $
#
# Copyright (c) 2006-2011 Otto-von-Guericke-UniversitŠt Magdeburg
#
# This file is part of ECLecture.
#
__author__ = """Mario Amelung <mario.amelung@gmx.de>"""
__docformat__ = 'plaintext'

import re

from zope.interface import implements

from Products.validation import validation
from Products.validation.interfaces.IValidator import IValidator

TIME_PERIOD_VALIDATOR_NAME = 'isTimePeriod'

class TimePeriodValidator:
    """
    Ensure that we don't get a value for start and/or end time of a time period
    which are not valid.
    """

    #__implements__ = IValidator
    implements(IValidator)
    
    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description
    
    def __call__(self, value, *args, **kwargs):
        """
        """
        if value[0] or value[1]:

            for item in value:
                m = re.match('^\s*(\d\d)[.:]?(\d\d)\s*$', item)
    
                if not m:
                    return ("Validation failed: '%(value)s' is not a time specification." %
                            { 'value': item, })
        
        return True

# register time period validator 
isTimePeriod = TimePeriodValidator(TIME_PERIOD_VALIDATOR_NAME)
validation.register(isTimePeriod)



# -*- coding: utf-8 -*-
# $Id: __init__.py 1542 2011-04-01 07:13:53Z amelung $
#
# Copyright (c) 2006-2011 Otto-von-Guericke-UniversitŠt Magdeburg
#
# This file is part of ECLecture.
#
__author__ = """Mario Amelung <mario.amelung@gmx.de>"""
__docformat__ = 'plaintext'


import os
import logging

from zope.i18nmessageid import MessageFactory

from Products.Archetypes import listTypes
from Products.Archetypes.atapi import process_types
from Products.Archetypes.utils import capitalize
from Products.CMFCore import DirectoryView
from Products.CMFCore import permissions as cmfpermissions
from Products.CMFCore import utils as cmfutils
from Products.CMFPlone.utils import ToolInit


from Products.ECLecture import config 

LOG = logging.getLogger(config.PROJECTNAME)

DirectoryView.registerDirectory('skins', config.product_globals)

ECMessageFactory = MessageFactory('eduComponents')


def initialize(context):
    """initialize product (called by zope)
    """

    # imports packages and types for registration
    import content

    # Initialize portal content
    all_content_types, all_constructors, all_ftis = process_types(
        listTypes(config.PROJECTNAME),
        config.PROJECTNAME)

    cmfutils.ContentInit(
        config.PROJECTNAME + ' Content',
        content_types      = all_content_types,
        permission         = config.DEFAULT_ADD_CONTENT_PERMISSION,
        extra_constructors = all_constructors,
        fti                = all_ftis,
        ).initialize(context)

    # Give it some extra permissions to control them on a per class limit
    for i in range(0,len(all_content_types)):
        klassname=all_content_types[i].__name__
        if not klassname in config.ADD_CONTENT_PERMISSIONS:
            continue

        context.registerClass(meta_type   = all_ftis[i]['meta_type'],
                              constructors= (all_constructors[i],),
                              permission  = config.ADD_CONTENT_PERMISSIONS[klassname])





# -*- coding: utf-8 -*-
# $Id: portlet.py 1542 2011-04-01 07:13:53Z amelung $
#
# Copyright (c) 2006-2011 Otto-von-Guericke-UniversitŠt Magdeburg
#
# This file is part of ECLecture.
#
__author__ = """Mario Amelung <mario.amelung@gmx.de>"""
__docformat__ = 'plaintext'

#from Acquisition import aq_inner

from zope.component import getMultiAdapter#, ComponentLookupError
from zope.interface import implements

from plone.portlets.interfaces import IPortletDataProvider
from plone.app.portlets.portlets import base
from plone.memoize.view import memoize

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from Products.ECLecture import ECMessageFactory as _

class IECLecturePortlet(IPortletDataProvider):
    """A portlet

    It inherits from IPortletDataProvider because for this portlet, the
    data that is being rendered and the portlet assignment itself are the
    same.
    """
    pass


class Assignment(base.Assignment):
    """
    Portlet assignment.

    This is what is actually managed through the portlets UI and associated
    with columns.
    """
    implements(IECLecturePortlet)

    def __init__(self):
        pass

    @property
    def title(self):
        """
        This property is used to give the title of the portlet in the
        'manage portlets' screen.
        """
        return _(u"Today's Lectures")


class Renderer(base.Renderer):
    """
    Portlet renderer.

    This is registered in configure.zcml. The referenced page template is
    rendered, and the implicit variable 'view' will refer to an instance
    of this class. Other methods can be added and referenced in the template.
    """

    render = ViewPageTemplateFile('portlet.pt')

    def __init__(self, *args):
        """
        """
        base.Renderer.__init__(self, *args)
        
        tools = getMultiAdapter((self.context, self.request),
                                name=u'plone_tools')

        self.catalog = tools.catalog()

        portal_state = getMultiAdapter((self.context, self.request),
                                       name=u'plone_portal_state')

        self.portal = portal_state.portal()
        self.portal_url = portal_state.portal_url()

    @property
    def available(self):
        """
        Determine if the portlet is available at all.
        """
        if self.brains():
            return True

    @memoize
    def brains(self):
        """
        """
        return self.catalog.searchResults(
                                portal_type='ECLecture',
                                end={'query':self.context.ZopeTime(), 'range':'min'},
                                sort_on='getTimePeriod',
                                review_state='published')

    @memoize
    def lectures(self):
        """
        """
        lectureObjects = map(lambda (item): item.getObject(), self.brains());
        
        return filter(lambda (item): item.lectureTakesPlace(), lectureObjects)[:5]



class AddForm(base.NullAddForm):
    """Portlet add form.

    This is registered in configure.zcml. The form_fields variable tells
    zope.formlib which fields to display. The create() method actually
    constructs the assignment that is being added.
    """

    def create(self):
        return Assignment()



# -*- coding: utf-8 -*-
# $Id: setuphandlers.py 1568 2011-06-28 22:31:28Z amelung $
#
# Copyright (c) 2006-2011 Otto-von-Guericke-UniversitŠt Magdeburg
#
# This file is part of ECLecture.
#
__author__ = """Mario Amelung <mario.amelung@gmx.de>"""
__docformat__ = 'plaintext'

import transaction

from Products.CMFCore.utils import getToolByName

from Products.ECLecture import config
from Products.ECLecture import LOG


def isNotECLectureProfile(context):
    return context.readDataFile("ECLecture_marker.txt") is None


def updateRoleMappings(context):
    """after workflow changed update the roles mapping. this is like pressing
    the button 'Update Security Setting' and portal_workflow"""
    if isNotECLectureProfile(context): return 
    wft = getToolByName(context.getSite(), 'portal_workflow')
    wft.updateRoleMappings()


def postInstall(context):
    """Called as at the end of the setup process. """
    # the right place for your custom code
    if isNotECLectureProfile(context): return

    #site = context.getSite()
    reindexIndexes(context)


def installGSDependencies(context):
    """Install dependend profiles."""
    
    # Has to be refactored as soon as generic setup allows a more 
    # flexible way to handle dependencies.
    
    return


def installQIDependencies(context):
    """Install dependencies"""
    
    if isNotECLectureProfile(context): return

    site = context.getSite()

    portal = getToolByName(site, 'portal_url').getPortalObject()
    quickinstaller = portal.portal_quickinstaller
    for dependency in config.DEPENDENCIES:
        LOG.info('Installing dependency %s:' % dependency)
        quickinstaller.installProduct(dependency)
        transaction.savepoint() 


def reindexIndexes(context):
    """Reindex some indexes.

    Indexes that are added in the catalog.xml file get cleared
    everytime the GenericSetup profile is applied.  So we need to
    reindex them.

    Since we are forced to do that, we might as well make sure that
    these get reindexed in the correct order.
    """
    if isNotECLectureProfile(context): return 

    site = context.getSite()

    pc = getToolByName(site, 'portal_catalog')
    indexes = [
        'getTimePeriod',
        ]

    # Don't reindex an index if it isn't actually in the catalog.
    # Should not happen, but cannot do any harm.
    ids = [id for id in indexes if id in pc.indexes()]
    if ids:
        pc.manage_reindexIndex(ids=ids)
    
    LOG.info('Indexes %s re-indexed' % indexes)



# -*- coding: utf-8 -*-
# $Id: ECLecture.py 1571 2011-08-10 17:44:43Z amelung $
#
# Copyright (c) 2006-2011 Otto-von-Guericke-UniversitŠt Magdeburg
#
# This file is part of ECLecture.
#
__author__ = 'Mario Amelung <mario.amelung@gmx.de>'
__docformat__ = 'plaintext'

#import re
#import urllib
import interfaces
from types import StringType, IntType
from DateTime import DateTime
from AccessControl import ClassSecurityInfo
from zope.interface import implements

from Products.CMFCore.utils import getToolByName

from Products.Archetypes.atapi import Schema, DisplayList, registerType
from Products.Archetypes.atapi import StringField, LinesField, DateTimeField, \
    IntegerField, TextField
from Products.Archetypes.atapi import StringWidget, LinesWidget, \
    CalendarWidget, SelectionWidget, TextAreaWidget, RichWidget

from Products.CMFCore import permissions

from Products.DataGridField.DataGridField import DataGridField
from Products.DataGridField.DataGridWidget import DataGridWidget


from Products.ECLecture import config
from Products.ECLecture import LOG
from Products.ECLecture import ECMessageFactory as _
from Products.ECLecture.content.TimePeriodField import TimePeriodField

try:
    from Products.ECAssignmentBox.content.ECFolder import ECFolder as SuperClass
    from Products.ECAssignmentBox.content.ECFolder import ECFolder_schema as SuperSchema
except:
    LOG.debug('Import failed: Products.ECAssignmentBox.content.ECFolder')
    from Products.ATContentTypes.content.folder import ATFolder as SuperClass
    from Products.ATContentTypes.content.folder import ATFolderSchema as SuperSchema


NO_RECURRENCE = "0"
DAILY = "1"
WEEKLY = "2"
MONTHLY = "3"
YEARLY = "4"

NO_GROUP = ""


schema = Schema((

    StringField('courseType',
        required = False,
        widget = StringWidget(
            label = "Course type",
            description = "Enter the type of this course (e.g., Lecture or Lab Exercise)",
            label_msgid = 'label_course_type',
            description_msgid = 'help_course_type',
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),

    LinesField('instructors',
        required = True,
        languageIndependent = True,
        searchable = True,
        widget = LinesWidget(
            label = "Instructors",
            description = "User names or names of instructors, one per line",
            label_msgid = 'label_instructors',
            description_msgid = 'help_instructors',
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),

    TimePeriodField('timePeriod',
        accessor = 'getTimePeriod',
        edit_accessor = 'getTimePeriodForEdit',
        required = True,
        default = ['11:00', '13:00'],
        widget = StringWidget(
            macro = 'time_period',
            size = 5,
            maxlength = 5,
            label = "Time period",
            description = "Start and end times of this course",
            label_msgid = 'label_time_period',
            description_msgid = 'help_time_period',
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),

    DateTimeField('startDate',
        required = True,
        widget = CalendarWidget(
            label = "Start date",
            description = "First regular date",
            label_msgid = 'label_start_date',
            description_msgid = 'help_start_date',
            show_hm = False, 
            #show_ymd = True,
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),
    
    DateTimeField('endDate',
        required = True,
        widget = CalendarWidget(
            label = "End date",
            description = "Last regular date",
            label_msgid = 'label_end_date',
            description_msgid = 'help_end_date',
            show_hm = False, 
            #show_ymd = True,
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),
                                                   
    StringField('recurrence',
        required = True,
        vocabulary = 'getRecurrenceDisplayList',
        default = WEEKLY,
        widget = SelectionWidget(
            format = "radio", # possible values: flex, select, radio
            label = "Recurrence",
            description = "How often this course takes place",
            label_msgid = 'label_recurrence',
            description_msgid = 'help_recurrence',
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),
                                                   
    DateTimeField('firstSession',
        widget = CalendarWidget(
            label = "First session",
            description = "Date for the first session for this course",
            label_msgid = 'label_first_session',
            description_msgid = 'help_first_session',
            #show_hm = False, 
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),

    StringField('location',
        required = True,
        widget = StringWidget(
            label = "Location",
            description = "Location for this course",
            label_msgid = 'label_location',
            description_msgid = 'help_location',
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),

    StringField('courseLanguage',
        vocabulary = 'getLanguagesDL',
        widget = SelectionWidget(
            label = 'Language of instruction',
            description = 'The language used for teaching this course',
            label_msgid = 'label_course_language',
            description_msgid = 'help_course_language',
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),

    StringField('credits',
        required = False,
        widget = StringWidget(
            label = "Credits",
            description = "Credits which can be gained in this course",
            label_msgid = 'label_credits',
            description_msgid = 'help_credits',
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),

    TextField('prereq',
        required = False,
        default_content_type = config.DEFAULT_CONTENT_TYPE,
        default_output_type = config.DEFAULT_OUTPUT_TYPE,
        allowable_content_types = config.ALLOWED_CONTENT_TYPES,
        widget = TextAreaWidget(
            label = "Prerequisites",
            description = "Describe which prerequisites are required for this course",
            label_msgid = 'label_prereq',
            description_msgid = 'help_prereq',
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),

    TextField('target',
        required = False,
        default_content_type = config.DEFAULT_CONTENT_TYPE,
        default_output_type = config.DEFAULT_OUTPUT_TYPE,
        allowable_content_types = config.ALLOWED_CONTENT_TYPES,
        widget = TextAreaWidget(
            label = "Target group",
            description = "Describe for which audience this course is intended",
            label_msgid = 'label_target',
            description_msgid = 'help_target',
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),

    IntegerField('maxParticipants',
        required = False,
        widget = StringWidget(
            label = "Maximum number of participants",
            size = 4,
            description = "If there is an enrollment limit, specify the maximum number of participants",
            label_msgid = 'label_max_participants',
            description_msgid = 'help_max_participants',
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),

    StringField('joinURL',
        required = False,
        widget = StringWidget(
            label = "Registration link",
            description = "Link to the registration for this course. Use 'ecl_register' to let ECLecture handle enrollments for this course.",
            label_msgid = 'label_join_url',
            description_msgid = 'help_join_url',
            size = 65,
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),

    StringField('directoryEntry',
        required = False,
        widget = StringWidget(
            label = "Directory entry",
            description = "Link to the directory entry for this course.",
            label_msgid = 'label_directory_entry',
            description_msgid = 'help_directory_entry',
            size = 65,
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),

    StringField('associatedGroup',
        required = False,
        vocabulary = 'getGroupsDisplayList',
        default = NO_GROUP,
        widget = SelectionWidget(
            format = "select", # possible values: flex, select, radio
            label = "Associated group",
            description = """You can associate a group with this course to 
represent its participants; necessary if 'ecl_register' is used for enrollment 
management.""",
            label_msgid = 'label_associated_group',
            description_msgid = 'help_associated_group',
            i18n_domain = config.I18N_DOMAIN,
        ),
    ),

    DataGridField('availableResources',
        searchable = True,
        #default_method = 'getDefaultResources',
        #required = True,
        columns = ('title', 'url'),
        allow_empty_rows = False,
        widget = DataGridWidget(
            label_msgid = 'label_available_resourcess',
            label = "Available resources",
            description_msgid = 'help_available_resources',
            description = """Enter available resources for this course. Title 
is the name of a resource as shown to the user, URL must be a path inside
this site or an URL to an external source. Please remember that published 
items inside this course are added by default.""",
            column_names = ('Title', 'URL',),
            i18n_domain =  config.I18N_DOMAIN,
        ),
    ),

    TextField('text',
        #required=True,
        searchable=True,
        primary=True,
        #storage = AnnotationStorage(migrate=True),
        validators = ('isTidyHtmlWithCleanup',),
        #validators = ('isTidyHtml',),
        default_content_type = config.DEFAULT_CONTENT_TYPE,
        default_output_type = config.DEFAULT_OUTPUT_TYPE,
        allowable_content_types = config.ALLOWED_CONTENT_TYPES,
        widget = RichWidget(
            label = "Body Text",
            label_msgid = "label_body_text",
            description = "Enter course information",
            description_msgid = "help_body_text",
            rows = 18,
            i18n_domain = config.I18N_DOMAIN,
            allow_file_upload = config.ALLOW_DOCUMENT_UPLOAD,

        )
    ),

),
)


ECLecture_schema = SuperSchema.copy() + schema.copy()

if 'directions' in ECLecture_schema:
    # hide directions field if inheriting from ECFolder
    ECLecture_schema['directions'].widget.visible = {'view' : 'invisible',
                                           'edit' : 'invisible' }
    # move inherited fields to separate edit page
    ECLecture_schema['completedStates'].schemata = 'more'
    ECLecture_schema['projectedAssignments'].schemata = 'more'


class ECLecture(SuperClass):
    """A folder which contains lecture information."""
    security = ClassSecurityInfo()

    implements(interfaces.IECLecture)

    meta_type = 'ECLecture'
    _at_rename_after_creation = True

    schema = ECLecture_schema


    # Methods
    security.declarePublic('isECFolderish')
    def isECFolderish(self):
        """
        @return: True, if ECAssignmentBox is installed and ECLecture is 
                 derived from ECFolder    
        """
        # check whether or not ECAssignmentBox is installed
        qi = getToolByName(self, 'portal_quickinstaller')
        return  qi.isProductInstalled('ECAssignmentBox')

    
    security.declarePublic('getRecurrenceDisplayList')
    def getRecurrenceDisplayList(self):
        """
        Returns a display list of recurrence types.
        """
        dl = DisplayList(())
        
        dl.add(NO_RECURRENCE, _(u'once', default=u'once'))
        dl.add(DAILY, _(u'daily', default=u'daily'))
        dl.add(WEEKLY, _(u'weekly', default=u'weekly'))
        dl.add(MONTHLY, _(u'monthly',default=u'monthly'))
        #dl.add(YEARLY, _(u'yearly', default=u'yearly'))

        return dl

    security.declarePublic('getGroupsDisplayList')
    def getGroupsDisplayList(self):
        """
        Return all available groups as a display list.
        """
        dl = DisplayList(())
        dl.add(NO_GROUP, '----')

        # portal_groups.listGroups is deprecated and will be removed in 
        # Plone 3.5. Use PAS searchGroups instead.
        #pas = getToolByName(self, 'acl_users')
        #groups = pas.searchGroups() 
        
        groups_tool = getToolByName(self, 'portal_groups')
        groups = groups_tool.listGroups()

        #LOG.debug('getGroupsDisplayList: groups: %s' % groups)

        for group in groups:
            dl.add(group.getGroupId(), group.getGroupName())
        
        return dl


    security.declareProtected(permissions.View, 'getLanguagesDL')
    def getLanguagesDL(self):
        """
        Vocabulary method for the courseLanguage field.

        This method is based on languages() in ExtensibleMetadata.py. 
        availableLanguages() is defined in a Script (Python)
        (CMFPlone/skins/plone_scripts/availableLanguages.py)
        """
        available_langs = getattr(self, 'availableLanguages', None)
        if available_langs is None:
            return DisplayList(
                (('en','English'), ('de','German'), ('fr','French'),
                 ('es','Spanish'), ('pt','Portuguese'), ('ru','Russian')))
        if callable(available_langs):
            available_langs = available_langs()
        return DisplayList(available_langs)


    def getGroupMembers(self, groupname):
        """
        This is a horrible workaround for the silly and totally
        unnecessary (the code is already there!) limitation that you
        can't retrieve the group members for groups stored in LDAP.

        Returns a list of member objects.
        
        @deprecated in Plone 3 we shouldn't need this work-around 
        """
        mtool = getToolByName(self, 'portal_membership')
        #gtool = getToolByName(self, 'portal_groups')
        #groups = gtool.listGroupIds()
        
        # listGroupIds will be removed in Plone 3.5
        # we should use searchGroups instead
        #groups = self.acl_users.searchGroups()

        members = []

        if groupname:
            for userid in mtool.listMemberIds():
                if userid:
                    member = mtool.getMemberById(userid)
                    #if 'group_' + groupname in member.getGroups():
                    if str(groupname) in member.getGroups():
                        members.append(member)
            # end for
        # end if

        return members


    def getGroupMembersMailto(self, groupMembers, type=None):
        """
        Return a mailto: link with the e-mail addresses of the users
        given in groupMembers (a list of user names). If type is
        'bcc', create a link that contains the user in the To: header
        and the participants in the Bcc: header.
        """
        currentUser = self.portal_membership.getAuthenticatedMember()
        currentEmail = currentUser.getProperty('email')
        addresses = []

        for user in groupMembers:
            email = user.getProperty('email')
            if email != currentEmail and email not in addresses:
                addresses.append(email)

        if type == 'bcc':
            return 'mailto:?to=%s&bcc=%s' % (currentEmail, ','.join(addresses))
        else:
            return 'mailto:%s' % ','.join(addresses)

        #return 'mailto:' + ','.join([urllib.quote(user.getProperty('email'))
        #                      for user in groupMembers])


    security.declarePublic('isParticipant')
    def isParticipant(self, user_id):
        """
        Returns true, if the associated group contains the given 'user_id'. 
        """
        
        """
        examples from the Web:
        
            member and (member.id in here.portal_groups.getGroupById('group_name').getAllGroupMemberIds())

            membership = getToolByName(self, 'portal_membership')
            if membership.getMemberById(id) is not None:
                return 0
            groups = getToolByName(self, 'portal_groups')
            if groups.getGroupById(id) is not None:
                return 0
        """
        LOG.debug('here we are in ECLecture#isParticipant')

        groups_tool = getToolByName(self, 'portal_groups')
        
        if groups_tool:
            group = groups_tool.getGroupById(str(self.associatedGroup))
        
            if group:
                return (user_id in group.getAllGroupMemberIds())
        #end if
    
        return False


    security.declarePublic('getCurrentParticipants')
    def getCurrentParticipants(self):
        """
        Returns the number of user in the associated group.
        """
        LOG.debug('here we are in ECLecture#getCurrentParticipants')

        groups_tool = getToolByName(self, 'portal_groups')
        
        if groups_tool:
            group = groups_tool.getGroupById(str(self.associatedGroup))
        
            if group:
                return group.getAllGroupMemberIds()
        #end if
        
        return []
        

    security.declarePublic('hasEnrollmentLimitReached')
    def hasEnrollmentLimitReached(self):
        """
        Returns wether or not a user can enroll in this course due to the
        enrollment limit (maxParticipants).
        """
        LOG.debug('here we are in ECLecture#hasEnrollmentLimitReached')

        max = self.getMaxParticipants();
        current = len(self.getCurrentParticipants());
        
        if max: 
            result = not (current < max)
        else:
            result = False
              
        return result
    

    security.declarePublic('addParticipant')
    def addParticipant(self, user_id):
        """
        Add a user to the group associated with this lecture.
        """
        #LOG.info("xdebug: Adding user '%s' to '%s'" % (user_id, self.associatedGroup))

        group = self.acl_users.getGroupByName(str(self.associatedGroup))

        if group:
            try:
                group.addMember(user_id)
            except ValueError, ve:
                # This could happen for users who are not members of the
                # Plone site (e.g., admin)
                LOG.warn('Could not add participant: %s' % ve)
                return False

        else:
            raise Exception('%s is not a valid group.' % str(self.associatedGroup))
            
        return True
            

    security.declarePublic('removeParticipant')
    def removeParticipant(self, user_id):
        """
        Remove a user from the group associated with this lecture.
        """
        LOG.debug('here we are in ECLecture#removeParticipant: %s' % user_id)

        group = self.associatedGroup

        if group:
            try:
                self.acl_users.getGroupByName(group).removeMember(user_id)
            except ValueError, ve:
                # This could happen for users who are not members of the
                # Plone site (e.g., admin)
                LOG.warn('removeParticipant: %s', ve)
                return False
            return True


    security.declarePublic('getTimePeriod')
    def getTimePeriod(self):
        """
        @return a string representing a time period
        """
        value = self.getTimePeriodForEdit()
        return '–'.join(value)


    security.declarePublic('getStartDateWeekday')
    def getStartDateWeekday(self):
        """
        """
        if self.recurrence in [NO_RECURRENCE, WEEKLY]:
            date = self.getField('startDate').get(self)
            if date:
                return date.Day()


    security.declarePrivate('getTimePeriodForEdit')
    def getTimePeriodForEdit(self):
        """
        @return a list with two values representing start and end time of a 
                time period
        """
        value = self.getField('timePeriod').get(self)
        result = []

        try:
            for item in value:
                if type(item) is IntType:
                    result.append('%02d:%02d' % (item/60, item%60))
                elif type(item) is StringType:
                    # When the object is created, this can be the
                    # default value (a string)
                    result.append(item)
        except:
            raise Exception(repr(item))
            
        return result
    
    
#    def getDefaultResources(self):
#        """
#        """
#        slidesTitle = self.translate(domain=I18N_DOMAIN,
#                                     msgid='default_slides_title',
#                                     default='Slides')
#        
#        assignmentsTitle = self.translate(domain=I18N_DOMAIN,
#                                          msgid='default_assignments_title',
#                                          default='Assignments')
#
#        return ({'title':slidesTitle, 'url':'slides', 
#                'icon':'book_icon.gif'},                    
#               {'title':assignmentsTitle, 'url':'assignments', 
#                'icon':'topic_icon.gif'},
#        )

    security.declarePublic('start')
    def start(self):
        """
        Method providing an Event-like interface (works with
        CalendarX, for example).
        """
        date = getattr(self, 'startDate', None)
        return date is None and self.created() or date

    security.declarePublic('end')
    def end(self):
        """
        Method providing an Event-like interface (works with
        CalendarX, for example).
        """
        date = getattr(self, 'endDate', None)
        return date is None and self.start() or date    

    security.declarePublic('lectureTakesPlace')
    def lectureTakesPlace(self, datetime=None):
        """
        Return True if the lecture takes place on the given date. If
        no date is specified, the current date will be used.

        TODO: Currently not implemented for monthly and yearly recurrence.
        """
        
        result = False
        if not datetime:
            datetime = DateTime()

        if datetime >= self.startDate.earliestTime() \
               and datetime <= self.endDate.latestTime():
            if self.recurrence == NO_RECURRENCE:
                result = self.startDate.isCurrentDay()
            elif self.recurrence == DAILY:
                result = True
            elif self.recurrence == WEEKLY:
                result = datetime.dow() == self.startDate.dow()
            elif self.recurrence == MONTHLY:
                # TODO
                result = False
            elif self.recurrence == YEARLY:
                # TODO
                result = False
        return result


registerType(ECLecture, config.PROJECTNAME)
# end of class ECLecture



## Script (Python) "ecl_participants_export"
##parameters=format='tab'
##title=
##

# -*- coding: iso-8859-1 -*-
# $Id: ecl_participants_export.py 1265 2009-09-24 10:55:23Z amelung $
#
# Copyright © 2007 Otto-von-Guericke-Universität Magdeburg
#
# This file is part of ECLecture.
#
# ECLecture is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# ECLecture is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ECLecture; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

REQUEST  = context.REQUEST
RESPONSE = REQUEST.RESPONSE

from AccessControl import getSecurityManager

try:
    ecab_utils = context.ecab_utils
except:
    ecab_utils = None

I18N_DOMAIN='eduComponents'

target = context.getActionInfo('object/ecl_participants')['url']
REDIRECT_URL = '%s?portal_status_message=' % target

# Unless some participants were explicitly selected, export all
# participants
filter = False
if REQUEST.has_key('ids'):
    filter = True

# Define the export format
exportFormat   = ['tab', '\t', '\n', '"', '"', '"']
colDelim       = exportFormat[1]
rowDelim       = exportFormat[2]
strStart       = exportFormat[3]
strEnd         = exportFormat[4]
escapeChar     = exportFormat[5]
exportEncoding = 'iso-8859-15'
siteEncoding   = context.getCharset()

###############################################################################

def escape(string):
    escaped = ''
    for c in string:
        if(c in [escapeChar, strStart, strEnd]):
            escaped += escapeChar + c
        else:
            escaped += c
    return escaped.decode(siteEncoding).encode(exportEncoding)

###############################################################################

table  = []
output = colDelim.join(('fullname', 'username', 'email', 'personaltitle',
                       'studentid', 'major')) + rowDelim

# Create a table containing all required data
participants = context.getGroupMembers(context.getAssociatedGroup())

for p in participants:
    if (not filter) or (filter and p.id in REQUEST.ids):
        username = p.id
        
        if ecab_utils:
            fullname = ecab_utils.getFullNameById(username)
        else:
            fullname = p.fullname
        
        email = p.email
        row = [fullname, username, email,]

        if ecab_utils:
            props = context.portal_properties.ecab_properties
            studentid = ecab_utils.getUserPropertyById(username,
                                                  props.student_id_attr) or ""
            major     = ecab_utils.getUserPropertyById(username,
                                                       props.major_attr) or ""
            perstitle = ecab_utils.getUserPropertyById(username,
                                              props.personal_title_attr) or ""
            row.extend((perstitle, studentid, major,))
        
        table.append(row)

# Sort the table by full name
table.sort(lambda a, b: cmp(a[1], b[1]))

# Create the output
for row in table:
    for i in range(0, len(row)):
        col = row[i]
        if(same_type(col, '') or same_type(col, u'')):
            output += strStart + escape(col) + strEnd
        elif(same_type(col, 1.1)):
            if ecab_utils:
                output += escape(ecab_utils.localizeNumber("%.2f", col))
            else:
                output += escape("%.2f") % col
        else:
            output += escape(str(col))
        
        # Append column delimiter or row delimiter
        output += [colDelim, rowDelim][i >= (len(row) - 1)]

# Generate a filename; the object ID should already be filesystem-safe
filename = '%s_%s.%s' % (context.getId(), 'participants', format)

#RESPONSE.setHeader('Content-Disposition', 'inline') # Useful for debugging
RESPONSE.setHeader('Content-Disposition', 'attachment; filename=' + filename)
RESPONSE.setHeader('Content-Type', 'text/plain')
return output



# -*- coding: utf-8 -*-
# $Id: config.py 1568 2011-06-28 22:31:28Z amelung $
#
# Copyright (c) 2006-2011 Otto-von-Guericke-UniversitŠt Magdeburg
#
# This file is part of ECLecture.
#
__author__ = """Mario Amelung <mario.amelung@gmx.de>"""
__docformat__ = 'plaintext'

# Product configuration.
#
# The contents of this module will be imported into __init__.py, the
# workflow configuration and every content type module.
#
# If you wish to perform custom configuration, you may put a file
# AppConfig.py in your product's root directory. The items in there
# will be included (by importing) in this file if found.
from Products.CMFCore import permissions
from Products.ATContentTypes.config import zconf

product_globals = globals()

# Project
PROJECTNAME = "ECLecture"
# i18n 
I18N_DOMAIN = 'eduComponents'

# Dependencies of products to be installed by quick-installer
DEPENDENCIES = ['DataGridField']

# Dependend products - not quick-installed
PRODUCT_DEPENDENCIES = []

# supported mime types for textfields
#EC_MIME_TYPES = ('text/plain', 'text/structured', 'text/x-rst', 'text/x-web-intelligent', 'text/html', )
ALLOWED_CONTENT_TYPES = zconf.ATDocument.allowed_content_types

# default mime type for textfields
#EC_DEFAULT_MIME_TYPE = 'text/plain'
DEFAULT_CONTENT_TYPE = zconf.ATDocument.default_content_type

# default output format of textfields
DEFAULT_OUTPUT_TYPE = 'text/x-html-safe'
#DEFAULT_OUTPUT_TYPE = zconf.ATDocument.default_content_type

ALLOW_DOCUMENT_UPLOAD = zconf.ATDocument.allow_document_upload


# Permissions
DEFAULT_ADD_CONTENT_PERMISSION = "Add portal content"
ADD_CONTENT_PERMISSIONS = {'ECLecture': 'ECLecture: Add ECLecture', }
permissions.setDefaultRoles(DEFAULT_ADD_CONTENT_PERMISSION, ('Manager', 'Owner'))
permissions.setDefaultRoles('ECLecture: Add ECLecture', ('Manager','Owner'))



# -*- coding: utf-8 -*-
# $Id: __init__.py 1542 2011-04-01 07:13:53Z amelung $
#
# Copyright (c) 2006-2011 Otto-von-Guericke-UniversitŠt Magdeburg
#
# This file is part of ECLecture.
#
import ECLecture



# -*- coding: utf-8 -*-
# $Id: __init__.py 1542 2011-04-01 07:13:53Z amelung $
#
# Copyright (c) 2006-2011 Otto-von-Guericke-UniversitŠt Magdeburg
#
# This file is part of ECLecture.
#

# See http://peak.telecommunity.com/DevCenter/setuptools

#namespace-packages
try:
    __import__('pkg_resources').declare_namespace(__name__)
except ImportError:
    from pkgutil import extend_path
    __path__ = extend_path(__path__, __name__)



# -*- coding: utf-8 -*-
# $Id: setup.py 1542 2011-04-01 07:13:53Z amelung $
#
# This file is part of ECLecture.
#
import os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = read('Products', 'ECLecture', 'version.txt').strip()
readme  = read('Products', 'ECLecture', 'README.txt')
history = read('Products', 'ECLecture', 'CHANGES.txt')

long_description = readme + '\n\n' + history

setup(name='Products.ECLecture',
      version=version,
      description = "Managing lectures, seminars and other courses.",
      long_description = long_description,

      # Get more strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords = '',
      author = 'Mario Amelung and Michael Piotrowski',
      author_email = 'mario.amelung@gmx.de and mxp@dynalabs.de',
      url = 'http://plone.org/products/eclecture/',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'Products.DataGridField >= 1.6',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



# -*- coding: utf-8 -*-
# $Id: TimePeriodField.py 1542 2011-04-01 07:13:53Z amelung $
#
# Copyright (c) 2006-2011 Otto-von-Guericke-UniversitŠt Magdeburg
#
# This file is part of ECLecture.
#
__author__ = """Mario Amelung <mario.amelung@gmx.de>"""
__docformat__ = 'plaintext'

import re

from AccessControl import ClassSecurityInfo

from Products.Archetypes.public import ObjectField
from Products.Archetypes.public import StringWidget
from Products.Archetypes.Registry import registerField

# Import validators first to ensure, that  
# isTimePeriod/TimePeriodValidator is registered
from Products.ECLecture.content import validators 


class TimePeriodField(ObjectField):
    """
    A field that stores a list of two integer values representing 
    a time period
    """

    #__implements__ = ObjectField.__implements__

    _properties = ObjectField._properties.copy()
    _properties.update({
        'type' : 'integer',
        'size' : '5',
        'widget' : StringWidget,
        'default' : [],
        'validators' : (validators.TIME_PERIOD_VALIDATOR_NAME),
        })

    security  = ClassSecurityInfo()

    security.declarePrivate('validate_required')
    def validate_required(self, instance, value, errors):
        """
        Tests if all elements in value are not None. If one is None a 
        error message will be returned.
        
        @see ObjectField.validate_required
        """
        result = True
        
        for item in value:
            if not item:
                result = False
                break

        return ObjectField.validate_required(self, instance, result, errors)

    security.declarePrivate('set')
    def set(self, instance, value, **kwargs):
        """
        Tests if all elements in value are numbers and save them as minutes.
        
        @see ObjectField.set
        """
        result = []
        
        for item in value:
            if self.required or item:
                m = re.match('^(\d\d)[.:]?(\d\d)$', item.strip())
                result.append((int(m.group(1)) * 60 ) + int(m.group(2)))
            else:
                result = []
                break
        
        ObjectField.set(self, instance, result, **kwargs)


registerField(TimePeriodField,
              title='TimePeriod',
              description=('Stores a list of two integer values representing '
                           'a time period')
    )
